#!/usr/bin/env python3

"""Information about this version release
"""

__version__ = '1.0.0'
__title__ = 'mc-mod-getter'
__description__ = 'Easy Download & Update Mods for Minecraft'
__user__ = 'mgutierrezz'
__url__ = f'https://github.com/{__user__}/{__title__}'
__author__ = 'Michael Gutierrez'
__author_email__ = 'Michael.Gutierrez2@marist.edu'
__license__ = 'MIT'
__copyright__ = 'Copyright 2021 Michael Gutierrez'
__keywords__ = 'Package'
__project_urls__ = {
    'Bugs': f'https://github.com/{__user__}/{__title__}/issues/',
    'Documentation': f'https://github.com/{__user__}/{__title__}/',
    'Source Code': __url__,
}


#!/usr/bin/env python3

from pkg_resources import require
from mc_mod_getter import __project_urls__
from mc_mod_getter.utils.ApiHandler import ApiHandler
from pathlib import Path
import logging
import click
import yaml


@click.command()
@click.option('--file','-f', help='Path to mod yaml file',type=click.Path(exists=True), required=True)
@click.option('--verbose','-v', help='Verbose mode', is_flag=True)
def main(file,verbose):
    # TODO: Possibly get yaml from gui but thats extra work :/
    mods_yaml = file

    if verbose:
        logging.basicConfig(level=logging.INFO, format='[%(asctime)s] %(message)s')

    logging.info(f'Using yaml file: {mods_yaml}')

    mods_dict = yaml.safe_load(open(mods_yaml, 'r')).items()
    
    for host, mods in mods_dict:
        download_dir = mods.get('mmc_dir', str(Path.home() / 'Downloads'))
        logging.info(f'Downloading mods to: {download_dir}')

        api_handler = ApiHandler(host, mods['version'], mods['loader'], download_dir, verbose)

        for mod in mods['mods']:
            api_handler.download_mod(mod)


if __name__ == '__main__':
    main()

    


#!/usr/bin/env python3

import os
import hashlib
import requests
import logging

class ApiHandler:
    def __init__(self,host,version,loader,mod_dir,verbose):
        self.host = host
        self.version = str(version)
        self.loader = loader
        self.__set_api_host()
        self.mod_dir = mod_dir
        self.verbose = verbose
        logging.info(f'API Handler details: {self}')

    def __repr__(self) -> str:
        return str(self.__dict__)


    def __set_api_host(self):
        api_hosts = {
            'modrinth':'https://api.modrinth.com/api/v1/mod'
        }
        self.api_host = api_hosts[self.host]


    def download_mod(self, mod_name: str):
        if self.host == 'modrinth':
            self.__download_modrinth(mod_name)


    def __download_modrinth(self, mod_name: str):
        mod_id = self.__get_modrinth_id(mod_name)
        mod = self.__filter_modrinth_version(mod_id)
        filename = mod['files'][0]['filename']
        jar_file = os.path.join(self.mod_dir, filename)

        logging.info(f'From:{self.host} Downloading:{mod_name}({mod_id}) File:{filename}')

        # Download the mod, if the file hashes dont match, redownload the mod and check again
        while True:
            try: 
                with open(jar_file, 'wb') as f:
                    f.write(requests.get(mod['files'][0]['url'], stream=True).content)  

                if not self.__file_checksum(jar_file, mod['files'][0]['hashes']['sha512']):
                    continue
            except:
                if not mod:
                    logging.info(f'{mod_name} not found, check on {self.host} if it exists or mod name spelling')
            else:
                break


    def __get_modrinth_id(self, mod_name: str) -> str:
        request =  f'{self.api_host}?query={mod_name.lower()}'
        
        for mod in requests.request('GET',request).json().get('hits'):
            if mod_name in mod.get('title') and self.loader in mod.get('categories'):
                return mod["mod_id"].split('-')[1]


    def __filter_modrinth_version(self, mod_id: str):
        # Send the id, and get back all version available for the mod
        request = f'{self.api_host}/{mod_id}/version'
        mod_versions = requests.request('GET', request).json()

        # Get all versions that match one in yaml file
        versions = [v for v in mod_versions if self.version == v['game_versions'][-1]]
        
        return versions[0] if versions else None


    def __file_checksum(self, file_path, url_hash) -> bool:

        with open(file_path, 'rb') as f:
            file_hash = hashlib.sha512()
            while chunk := f.read(8192):
                file_hash.update(chunk)

        return file_hash.hexdigest() == url_hash
